@@ -126,7 +126,8 @@ class MetadataTypeResource(object):
             return
 
         if not can_update:
-            full_message = "Unsafe changes at " + ", ".join(".".join(offset) for offset, _, _ in unsafe_changes)
+            full_message = "Unsafe changes at " + ", ".join(".".join(map(str, offset))
+                                                            for offset, _, _ in unsafe_changes)
             raise ValueError(full_message)
 
         _LOG.info("Updating metadata type %s", metadata_type.name)
@@ -193,7 +194,8 @@ class MetadataTypeResource(object):
             raise KeyError('%s is not a valid MetadataType name' % name)
         return self._make_from_query_row(record)
 
-    def check_field_indexes(self, allow_table_lock=False, rebuild_all=False):
+    def check_field_indexes(self, allow_table_lock=False, rebuild_all=None,
+                            rebuild_views=False, rebuild_indexes=False):
         """
         Create or replace per-field indexes and views.
         :param allow_table_lock:
@@ -202,8 +204,19 @@ class MetadataTypeResource(object):
 
             If false, creation will be slightly slower and cannot be done in a transaction.
         """
+        if rebuild_all is not None:
+            warnings.warn(
+                "The rebuild_all option of check_field_indexes() is deprecated.",
+                "Instead, use rebuild_views=True or rebuild_indexes=True as needed.",
+                DeprecationWarning)
+            rebuild_views = rebuild_indexes = rebuild_all
+
         with self._db.connect() as connection:
-            connection.check_dynamic_fields(concurrently=not allow_table_lock, rebuild_all=rebuild_all)
+            connection.check_dynamic_fields(
+                concurrently=not allow_table_lock,
+                rebuild_indexes=rebuild_indexes,
+                rebuild_views=rebuild_views,
+            )
 
     def get_all(self):
         """
@@ -540,6 +553,9 @@ class ProductResource(object):
                 if not field:
                     # This type doesn't have that field, so it cannot match.
                     break
+                if not hasattr(field, 'extract'):
+                    # non-document/native field
+                    continue
                 if field.extract(type_.metadata_doc) is None:
                     # It has this field but it's not defined in the type doc, so it's unmatchable.
                     continue
@@ -837,24 +853,44 @@ class DatasetResource(object):
             out.update(type_.metadata_type.dataset_fields)
         return out
 
-    def get_locations(self, dataset):
+    def get_locations(self, id_):
         """
-        :param datacube.model.Dataset dataset: dataset
+        :param typing.Union[UUID, str] id_: dataset id
         :rtype: list[str]
         """
-        with self._db.connect() as connection:
-            return connection.get_locations(dataset.id)
+        if isinstance(id_, Dataset):
+            warnings.warn("Passing dataset is deprecated after 1.2.2, pass dataset.id", DeprecationWarning)
+            id_ = id_.id
 
-    def add_location(self, dataset, uri):
+        with self._db.connect() as connection:
+            return connection.get_locations(id_)
+
+    def get_archived_locations(self, id_):
+        """
+        :param typing.Union[UUID, str] id_: dataset id
+        :rtype: list[str]
+        """
+        if isinstance(id_, Dataset):
+            warnings.warn("Passing dataset is deprecated after 1.2.2, pass dataset.id", DeprecationWarning)
+            id_ = id_.id
+
+        with self._db.connect() as connection:
+            return connection.get_archived_locations(id_)
+
+    def add_location(self, id_, uri):
         """
         Add a location to the dataset if it doesn't already exist.
-        :param datacube.model.Dataset dataset: dataset
+        :param typing.Union[UUID, str] id_: dataset id
         :param str uri: fully qualified uri
         :returns bool: Was one added?
         """
+        if isinstance(id_, Dataset):
+            warnings.warn("Passing dataset is deprecated after 1.2.2, pass dataset.id", DeprecationWarning)
+            id_ = id_.id
+
         with self._db.connect() as connection:
             try:
-                connection.ensure_dataset_location(dataset.id, uri)
+                connection.ensure_dataset_location(id_, uri)
                 return True
             except DuplicateRecordError:
                 return False
@@ -863,17 +899,51 @@ class DatasetResource(object):
         with self._db.connect() as connection:
             return (self._make(row) for row in connection.get_datasets_for_location(uri))
 
-    def remove_location(self, dataset, uri):
+    def remove_location(self, id_, uri):
         """
         Remove a location from the dataset if it exists.
-        :param datacube.model.Dataset dataset: dataset
+        :param typing.Union[UUID, str] id_: dataset id
         :param str uri: fully qualified uri
         :returns bool: Was one removed?
         """
+        if isinstance(id_, Dataset):
+            warnings.warn("Passing dataset is deprecated after 1.2.2, pass dataset.id", DeprecationWarning)
+            id_ = id_.id
+
         with self._db.connect() as connection:
-            was_removed = connection.remove_location(dataset.id, uri)
+            was_removed = connection.remove_location(id_, uri)
             return was_removed
 
+    def archive_location(self, id_, uri):
+        """
+        Archive a location of the dataset if it exists.
+        :param typing.Union[UUID, str] id_: dataset id
+        :param str uri: fully qualified uri
+        :return bool: location was able to be archived
+        """
+        if isinstance(id_, Dataset):
+            warnings.warn("Passing dataset is deprecated after 1.2.2, pass dataset.id", DeprecationWarning)
+            id_ = id_.id
+
+        with self._db.connect() as connection:
+            was_archived = connection.archive_location(id_, uri)
+            return was_archived
+
+    def restore_location(self, id_, uri):
+        """
+        Un-archive a location of the dataset if it exists.
+        :param typing.Union[UUID, str] id_: dataset id
+        :param str uri: fully qualified uri
+        :return bool: location was able to be restored
+        """
+        if isinstance(id_, Dataset):
+            warnings.warn("Passing dataset is deprecated after 1.2.2, pass dataset.id", DeprecationWarning)
+            id_ = id_.id
+
+        with self._db.connect() as connection:
+            was_restored = connection.restore_location(id_, uri)
+            return was_restored
+
     def _make(self, dataset_res, full_info=False):
         """
         :rtype datacube.model.Dataset
